@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 2007 - 2008 The GNOME Foundation.
+ * Copyright (C) 2007 - 2009 The GNOME Foundation.
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -34,12 +34,21 @@
 #include <libgda/gda-holder.h>
 #include <libgda/gda-log.h>
 #include <libgda/gda-util.h>
-#ifdef HAVE_FAM
-#include <fam.h>
-#include <glib.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
+#include <libgda/sqlite/gda-sqlite-provider.h>
+
+#ifdef HAVE_GIO
+  #ifdef HAVE_FAM
+    #error Impossible to have GIO and FAM at the same time
+  #endif
+  #include <gio/gio.h>
+#else
+  #ifdef HAVE_FAM
+    #include <fam.h>
+    #include <glib.h>
+    #include <sys/types.h>
+    #include <sys/stat.h>
+    #include <unistd.h>
+  #endif
 #endif
 #ifdef G_OS_WIN32 
 #include <io.h>
@@ -57,6 +66,7 @@ struct _GdaConfigPrivate {
 	gboolean system_config_allowed;
 	GSList *dsn_list; /* list of GdaDsnInfo structures */
 	GSList *prov_list; /* list of InternalProvider structures */
+	gboolean providers_loaded; /* TRUE if providers list has already been scanned */
 };
 
 static void gda_config_class_init (GdaConfigClass *klass);
@@ -101,9 +111,24 @@ enum {
 
 static GObjectClass *parent_class = NULL;
 
+#ifdef HAVE_GIO
+/*
+ * GIO static variables
+ */
+static GFileMonitor *mon_conf_user = NULL;
+static GFileMonitor *mon_conf_global = NULL;
+gulong user_notify_changes = 0;
+gulong global_notify_changes = 0;
+
+static void conf_file_changed (GFileMonitor *mon, GFile *file, GFile *other_file,
+			       GFileMonitorEvent event_type, gpointer data);
+static void lock_notify_changes (void);
+static void unlock_notify_changes (void);
+#endif
+
 #ifdef HAVE_FAM
 /*
- * FAM delcarations and static variables
+ * FAM declarations and static variables
  */
 static FAMConnection *fam_connection = NULL;
 static gint           fam_watch_id = 0;
@@ -123,6 +148,11 @@ static GStaticRecMutex gda_mutex = G_STATIC_REC_MUTEX_INIT;
 #define GDA_CONFIG_LOCK() g_static_rec_mutex_lock(&gda_mutex)
 #define GDA_CONFIG_UNLOCK() g_static_rec_mutex_unlock(&gda_mutex)
 
+/* GdaServerProvider for SQLite as a shortcut, available
+ * even if the SQLite provider is not installed
+ */
+GdaServerProvider *_gda_config_sqlite_provider = NULL;
+
 /*
  * GdaConfig class implementation
  * @klass:
@@ -134,6 +164,13 @@ gda_config_class_init (GdaConfigClass *klass)
 
 	parent_class = g_type_class_peek_parent (klass);
 
+	/**
+	 * GdaConfig::dsn-added
+	 * @conf: the #GdaConfig object
+	 * @new_dsn: a #GdaDsnInfo
+	 *
+	 * Gets emitted whenever a new DSN has been defined
+	 */
 	gda_config_signals[DSN_ADDED] =
                 g_signal_new ("dsn-added",
                               G_TYPE_FROM_CLASS (object_class),
@@ -142,6 +179,13 @@ gda_config_class_init (GdaConfigClass *klass)
                               NULL, NULL,
                               _gda_marshal_VOID__POINTER,
                               G_TYPE_NONE, 1, G_TYPE_POINTER);
+	/**
+	 * GdaConfig::dsn-to-be-removed
+	 * @conf: the #GdaConfig object
+	 * @old_dsn: a #GdaDsnInfo
+	 *
+	 * Gets emitted whenever a DSN is about to be removed
+	 */
 	gda_config_signals[DSN_TO_BE_REMOVED] =
                 g_signal_new ("dsn-to-be-removed",
                               G_TYPE_FROM_CLASS (object_class),
@@ -150,6 +194,13 @@ gda_config_class_init (GdaConfigClass *klass)
                               NULL, NULL,
                               _gda_marshal_VOID__POINTER,
                               G_TYPE_NONE, 1, G_TYPE_POINTER);
+	/**
+	 * GdaConfig::dsn-removed
+	 * @conf: the #GdaConfig object
+	 * @old_dsn: a #GdaDsnInfo
+	 *
+	 * Gets emitted whenever a DSN has been removed
+	 */
 	gda_config_signals[DSN_REMOVED] =
                 g_signal_new ("dsn-removed",
                               G_TYPE_FROM_CLASS (object_class),
@@ -158,6 +209,13 @@ gda_config_class_init (GdaConfigClass *klass)
                               NULL, NULL,
                               _gda_marshal_VOID__POINTER,
                               G_TYPE_NONE, 1, G_TYPE_POINTER);
+	/**
+	 * GdaConfig::dsn-changed
+	 * @conf: the #GdaConfig object
+	 * @dsn: a #GdaDsnInfo
+	 *
+	 * Gets emitted whenever a DSN's definition has been changed
+	 */
 	gda_config_signals[DSN_CHANGED] =
                 g_signal_new ("dsn-changed",
                               G_TYPE_FROM_CLASS (object_class),
@@ -171,15 +229,17 @@ gda_config_class_init (GdaConfigClass *klass)
         object_class->set_property = gda_config_set_property;
         object_class->get_property = gda_config_get_property;
 
-	/* To translators: DSN stands for Data Source Name, it's a named connection string defined in ~/.libgda/config */
+	/* To translators: DSN stands for Data Source Name, it's a named connection string defined in $XDG_DATA_HOME/libgda/config */
 	g_object_class_install_property (object_class, PROP_USER_FILE,
-                                         g_param_spec_string ("user-filename", _("File to use for per-user DSN list"), 
-							      NULL, NULL,
+                                         g_param_spec_string ("user-filename", NULL, 
+							      "File to use for per-user DSN list", 
+							      NULL, 
 							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	/* To translators: DSN stands for Data Source Name, it's a named connection string defined in $PREFIX/etc/libgda-4.0/config */
 	g_object_class_install_property (object_class, PROP_USER_FILE,
-                                         g_param_spec_string ("system-filename", _("File to use for system-wide DSN list"), 
-							      NULL, NULL,
+                                         g_param_spec_string ("system-filename", NULL,
+							      "File to use for system-wide DSN list", 
+							      NULL,
 							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	
 	object_class->constructor = gda_config_constructor;
@@ -197,6 +257,7 @@ gda_config_init (GdaConfig *conf, GdaConfigClass *klass)
 	conf->priv->system_config_allowed = FALSE;
 	conf->priv->prov_list = NULL;
 	conf->priv->dsn_list = NULL;
+	conf->priv->providers_loaded = FALSE;
 }
 
 static void
@@ -369,6 +430,9 @@ save_config_file (gboolean is_system)
 		xmlSetProp (entry, BAD_CAST "value", BAD_CAST (info->description));
 	}
 
+#ifdef HAVE_GIO
+	lock_notify_changes ();
+#endif
 #ifdef HAVE_FAM
         fam_lock_notify ();
 #endif
@@ -381,6 +445,9 @@ save_config_file (gboolean is_system)
                         g_warning ("Error saving config data to '%s'", unique_instance->priv->system_file);
 	}
 	fflush (NULL);
+#ifdef HAVE_GIO
+	unlock_notify_changes ();
+#endif
 #ifdef HAVE_FAM
         fam_unlock_notify ();
 #endif
@@ -418,20 +485,29 @@ gda_config_constructor (GType type,
 
 		/* define user and system dirs. if not already defined */
 		if (!user_file_set) {
-			if (g_get_home_dir ()) {
-				gchar *confdir, *conffile;
-				gboolean setup_ok = TRUE;
-				confdir = g_build_path (G_DIR_SEPARATOR_S, g_get_home_dir (), ".libgda", NULL);
-				conffile = g_build_filename (g_get_home_dir (), ".libgda", "config", NULL);
+			gchar *confdir, *conffile;
+			gboolean setup_ok = TRUE;
+			confdir = g_build_path (G_DIR_SEPARATOR_S, g_get_user_data_dir (), "libgda", NULL);
+			conffile = g_build_filename (confdir, "config", NULL);
 
-				if (!g_file_test (confdir, G_FILE_TEST_EXISTS)) {
-					if (g_mkdir (confdir, 0700))
-						{
-							setup_ok = FALSE;
-							g_warning (_("Error creating user specific "
-								     "configuration directory '%s'"), 
-								   confdir);
-						}
+			if (!g_file_test (confdir, G_FILE_TEST_EXISTS)) {
+				gchar *old_path;
+				old_path = g_build_path (G_DIR_SEPARATOR_S, g_get_home_dir (), ".libgda", NULL);
+				if (g_file_test (old_path, G_FILE_TEST_EXISTS)) {
+					/* using $HOME/.libgda because it exists */
+					g_free (confdir);
+					confdir = old_path;
+					g_free (conffile);
+					conffile = g_build_filename (confdir, "config", NULL);
+				}
+				else {
+					g_free (old_path);
+					if (g_mkdir_with_parents (confdir, 0700)) {
+						setup_ok = FALSE;
+						g_warning (_("Error creating user specific "
+							     "configuration directory '%s'"), 
+							   confdir);
+					}
 					if (setup_ok) {
 						gchar *str;
 						gchar *full_file;
@@ -475,7 +551,7 @@ gda_config_constructor (GType type,
 						g_free (str);
 					}
 				}
-				else if (!g_file_test (confdir, G_FILE_TEST_IS_DIR)) {
+				if (setup_ok && !g_file_test (confdir, G_FILE_TEST_IS_DIR)) {
 					setup_ok = FALSE;
 					g_warning (_("User specific "
 						     "configuration directory '%s' exists and is not a directory"), 
@@ -488,10 +564,20 @@ gda_config_constructor (GType type,
 				else
 					g_free (conffile);
 			}
+			else {
+				if (!g_file_test (confdir, G_FILE_TEST_IS_DIR)) {
+					g_warning (_("User specific "
+						     "configuration directory '%s' exists and is not a directory"), 
+						   confdir);
+				}
+				else
+					unique_instance->priv->user_file = conffile;
+				g_free (confdir);
+			}
 		}
 		if (!system_file_set) 
 			unique_instance->priv->system_file = gda_gbr_get_file_path (GDA_ETC_DIR, 
-										   LIBGDA_ABI_NAME, "config", NULL);
+										    LIBGDA_ABI_NAME, "config", NULL);
 		unique_instance->priv->system_config_allowed = FALSE;
 		if (unique_instance->priv->system_file) {
 			FILE *file;
@@ -503,6 +589,28 @@ gda_config_constructor (GType type,
 		}
 
 		/* Setup file monitoring */
+#ifdef HAVE_GIO
+		if (unique_instance->priv->user_file) {
+			GFile *gf;
+			gf = g_file_new_for_path (unique_instance->priv->user_file);
+			mon_conf_user = g_file_monitor_file (gf, G_FILE_MONITOR_NONE, NULL, NULL);
+			if (mon_conf_user)
+				g_signal_connect (G_OBJECT (mon_conf_user), "changed",
+						  G_CALLBACK (conf_file_changed), NULL);
+			g_object_unref (gf);
+		}
+
+		if (unique_instance->priv->system_file) {
+			GFile *gf;
+			gf = g_file_new_for_path (unique_instance->priv->system_file);
+			mon_conf_global = g_file_monitor_file (gf, G_FILE_MONITOR_NONE, NULL, NULL);
+			if (mon_conf_user)
+				g_signal_connect (G_OBJECT (mon_conf_global), "changed",
+						  G_CALLBACK (conf_file_changed), NULL);
+			g_object_unref (gf);
+		}
+#endif
+
 #ifdef HAVE_FAM
 		if (!fam_connection) {
 			/* FAM init */
@@ -513,7 +621,7 @@ gda_config_constructor (GType type,
 			g_print ("Using FAM to monitor configuration files changes.\n");
 #endif
 			fam_connection = g_malloc0 (sizeof (FAMConnection));
-			if (FAMOpen2 (fam_connection, "libgnomedb user") != 0) {
+			if (FAMOpen2 (fam_connection, "libgda user") != 0) {
 				g_print ("FAMOpen failed, FAMErrno=%d\n", FAMErrno);
 				g_free (fam_connection);
 				fam_connection = NULL;
@@ -601,7 +709,7 @@ GQuark gda_config_error_quark (void)
 }
 
 /**
- * gda_config_get_type
+ * gda_config_get_type:
  * 
  * Registers the #GdaConfig class on the GLib type system.
  * 
@@ -1041,7 +1149,7 @@ gda_config_get_provider_info (const gchar *provider_name)
 	if (!unique_instance)
 		gda_config_get ();
 
-	if (!unique_instance->priv->prov_list) 
+	if (!unique_instance->priv->providers_loaded) 
 		load_all_providers ();
 
 	if (!g_ascii_strcasecmp (provider_name, "MS Access")) {
@@ -1154,7 +1262,7 @@ gda_config_list_providers (void)
 	if (!unique_instance)
 		gda_config_get ();
 
-	if (!unique_instance->priv->prov_list) 
+	if (!unique_instance->priv->providers_loaded) 
 		load_all_providers ();
 
 	model = gda_data_model_array_new_with_g_types (5,
@@ -1253,6 +1361,14 @@ load_all_providers (void)
 		load_providers_from_dir (str, FALSE);
 		g_free (str);
 	}
+	unique_instance->priv->providers_loaded = TRUE;
+
+	/* find SQLite provider, and instanciate it if not installed */
+	_gda_config_sqlite_provider = gda_config_get_provider ("SQLite", NULL);
+	if (!_gda_config_sqlite_provider) {
+		_gda_config_sqlite_provider = (GdaServerProvider*) 
+			g_object_new (GDA_TYPE_SQLITE_PROVIDER, NULL);
+	}
 }
 
 static InternalProvider *
@@ -1455,6 +1571,7 @@ load_providers_from_dir (const gchar *dirname, gboolean recurs)
 #endif
 			}
 		}
+		g_free (path);
 		g_module_close (handle);
 	}
 
@@ -1518,6 +1635,61 @@ internal_provider_free (InternalProvider *ip)
 /*
  * File monitoring actions
  */
+#ifdef HAVE_GIO
+static void
+conf_file_changed (GFileMonitor *mon, GFile *file, GFile *other_file,
+		   GFileMonitorEvent event_type, gpointer data)
+{
+	g_assert (unique_instance);
+
+	if (event_type != G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT)
+		return;
+
+#ifdef GDA_DEBUG_NO
+	gboolean is_system = (mon == mon_conf_global) ? TRUE : FALSE;
+	g_print ("Reloading config files (%s config has changed)\n", is_system ? "global" : "user");
+	GSList *list;
+	for (list = unique_instance->priv->dsn_list; list; list = list->next) {
+		GdaDsnInfo *info = (GdaDsnInfo *) list->data;
+		g_print ("[info %p]: %s/%s\n", info, info->provider, info->name);
+	}
+#endif
+	while (unique_instance->priv->dsn_list) {
+		GdaDsnInfo *info = (GdaDsnInfo *) unique_instance->priv->dsn_list->data;
+		g_signal_emit (unique_instance, gda_config_signals[DSN_TO_BE_REMOVED], 0, info);
+		unique_instance->priv->dsn_list = g_slist_remove (unique_instance->priv->dsn_list, info);
+		g_signal_emit (unique_instance, gda_config_signals[DSN_REMOVED], 0, info);
+		data_source_info_free (info);
+	}
+	
+	lock_notify_changes ();
+	if (unique_instance->priv->system_file)
+		load_config_file (unique_instance->priv->system_file, TRUE);
+	if (unique_instance->priv->user_file)
+		load_config_file (unique_instance->priv->user_file, FALSE);
+	unlock_notify_changes ();
+}
+
+static void
+lock_notify_changes (void)
+{
+	if (user_notify_changes != 0)
+		g_signal_handler_block (mon_conf_user, user_notify_changes);
+	if (global_notify_changes != 0)
+		g_signal_handler_block (mon_conf_global, global_notify_changes);
+}
+
+static void
+unlock_notify_changes (void)
+{
+	if (user_notify_changes != 0)
+		g_signal_handler_unblock (mon_conf_user, user_notify_changes);
+	if (global_notify_changes != 0)
+		g_signal_handler_unblock (mon_conf_global, global_notify_changes);
+}
+
+#endif
+
 #ifdef HAVE_FAM
 static gboolean
 fam_callback (GIOChannel *source, GIOCondition condition, gpointer data)
