@@ -1,5 +1,5 @@
 /* GDA Berkeley-DB data model
- * Copyright (C) 1998 - 2008 The GNOME Foundation
+ * Copyright (C) 1998 - 2010 The GNOME Foundation
  *
  * AUTHORS:
  *      Laurent Sansonetti <lrz@gnome.org>
@@ -29,9 +29,14 @@
 #include <libgda/gda-data-model.h>
 #include <libgda/gda-data-model-bdb.h>
 #include <db.h>
+#include <gmodule.h>
 
 #define BDB_VERSION  (10000*DB_VERSION_MAJOR+100*DB_VERSION_MINOR+DB_VERSION_PATCH)
 
+static int (*libdb_db_create) (DB **, DB_ENV *, u_int32_t) = NULL;
+static char *(*libdb_db_strerror) (int) = NULL;
+static char *_db_strerror (int i);
+
 struct _GdaDataModelBdbPrivate {
 	gchar    *filename;
 	gchar    *db_name;
@@ -76,7 +81,7 @@ static void gda_data_model_bdb_get_property (GObject *object,
                                                 GParamSpec *pspec);
 
 /* GdaDataModel interface */
-static void                 gda_data_model_bdb_data_model_init (GdaDataModelClass *iface);
+static void                 gda_data_model_bdb_data_model_init (GdaDataModelIface *iface);
 static gint                 gda_data_model_bdb_get_n_rows      (GdaDataModel *model);
 static gint                 gda_data_model_bdb_get_n_columns   (GdaDataModel *model);
 static GdaColumn           *gda_data_model_bdb_describe_column (GdaDataModel *model, gint col);
@@ -100,7 +105,7 @@ static GObjectClass *parent_class = NULL;
  * Object init and dispose
  */
 static void
-gda_data_model_bdb_data_model_init (GdaDataModelClass *iface)
+gda_data_model_bdb_data_model_init (GdaDataModelIface *iface)
 {
         iface->i_get_n_rows = gda_data_model_bdb_get_n_rows;
         iface->i_get_n_columns = gda_data_model_bdb_get_n_columns;
@@ -155,11 +160,11 @@ gda_data_model_bdb_class_init (GdaDataModelBdbClass *klass)
         object_class->set_property = gda_data_model_bdb_set_property;
         object_class->get_property = gda_data_model_bdb_get_property;
         g_object_class_install_property (object_class, PROP_FILENAME,
-                                         g_param_spec_string ("filename", "DB file", NULL, NULL,
+                                         g_param_spec_string ("filename", NULL, "DB file", NULL,
                                                               G_PARAM_READABLE | G_PARAM_WRITABLE |
                                                               G_PARAM_CONSTRUCT_ONLY));
         g_object_class_install_property (object_class, PROP_DB_NAME,
-                                         g_param_spec_string ("db-name", "Name of the database", NULL, NULL,
+                                         g_param_spec_string ("db-name", NULL, "Name of the database", NULL,
                                                               G_PARAM_READABLE | G_PARAM_WRITABLE |
                                                               G_PARAM_CONSTRUCT_ONLY));
 
@@ -213,7 +218,7 @@ add_error (GdaDataModelBdb *model, const gchar *err)
 {
 	GError *error = NULL;
 
-        g_set_error (&error, 0, 0, err);
+        g_set_error (&error, 0, 0, "%s", err);
         model->priv->errors = g_slist_append (model->priv->errors, error);
 }
 
@@ -249,12 +254,46 @@ gda_data_model_bdb_get_type (void)
 		if (type == 0) {
 			type = g_type_register_static (G_TYPE_OBJECT, "GdaDataModelBdb", &info, 0);
 			g_type_add_interface_static (type, GDA_TYPE_DATA_MODEL, &data_model_info);
+
+			GModule *module;
+			gchar *err = NULL;
+
+			module = g_module_open (BDB_LIB, G_MODULE_BIND_LAZY);
+			if (!module)
+				err = g_strdup_printf (_("Could not load the Berkeley DB library: %s"),
+					   g_module_error ());
+			else {
+				if (!g_module_symbol (module, "db_create", (gpointer *) &libdb_db_create))
+					err = g_strdup_printf (_("Could not load the '%s' symbol from the "
+								 "Berkeley DB library"),
+							       "db_create");
+				else if (!g_module_symbol (module, "db_strerror", (gpointer *) &libdb_db_strerror))
+					err = g_strdup_printf (_("Could not load the '%s' symbol from the "
+								 "Berkeley DB library"),
+							       "db_strerror");
+			}
+			if (err) {
+				g_warning (err);
+				g_free (err);
+				libdb_db_create = NULL;
+				libdb_db_strerror = NULL;
+			}
+			else
+				g_module_make_resident (module);
 		}
 		g_static_mutex_unlock (&registering);
 	}
 	return type;
 }
 
+static char *_db_strerror (int i)
+{
+	if (libdb_db_strerror)
+		return libdb_db_strerror (i);
+	else
+		return _("Berkeley DB library not loaded");
+}
+
 static void
 update_number_of_rows (GdaDataModelBdb *model)
 {
@@ -272,7 +311,7 @@ update_number_of_rows (GdaDataModelBdb *model)
 #endif
 			 0);
 	if (ret) {
-		add_error (model, db_strerror (ret));
+		add_error (model, _db_strerror (ret));
 		model->priv->n_rows = 0;
 	}
 	else {
@@ -322,9 +361,13 @@ gda_data_model_bdb_set_property (GObject *object,
 		DB *dbp;
 
 		/* open database */
-		ret = db_create (&dbp, NULL, 0);
+		if (!libdb_db_create) {
+			add_error (model, _("Berkeley DB library not loaded"));
+			goto out;
+		}
+		ret = libdb_db_create (&dbp, NULL, 0);
 		if (ret) {
-			add_error (model, db_strerror (ret));
+			add_error (model, _db_strerror (ret));
 			goto out;
 		}
 		
@@ -338,14 +381,14 @@ gda_data_model_bdb_set_property (GObject *object,
 				 DB_UNKNOWN, /* autodetect DBTYPE */
 				 0, 0);
 		if (ret) {
-			add_error (model, db_strerror (ret));
+			add_error (model, _db_strerror (ret));
 			goto out;
 		}
 
 		/* get cursor */
 		ret = dbp->cursor (dbp, NULL, &dbpc, 0);
 		if (ret) {
-			add_error (model, db_strerror (ret));
+			add_error (model, _db_strerror (ret));
 			goto out;
 		}
 		model->priv->dbpc = dbpc;
@@ -363,7 +406,7 @@ gda_data_model_bdb_set_property (GObject *object,
                         column = gda_column_new ();
                         model->priv->columns = g_slist_append (model->priv->columns , column);
 			gda_column_set_name (column, "key");
-			gda_column_set_title (column, "key");
+			gda_column_set_description (column, "key");
 			gda_column_set_g_type (column, GDA_TYPE_BINARY);
 		}
 
@@ -380,7 +423,7 @@ gda_data_model_bdb_set_property (GObject *object,
                         column = gda_column_new ();
                         model->priv->columns = g_slist_append (model->priv->columns , column);
 			gda_column_set_name (column, "data");
-			gda_column_set_title (column, "data");
+			gda_column_set_description (column, "data");
 			gda_column_set_g_type (column, GDA_TYPE_BINARY);
 		}
 		model->priv->n_columns = g_slist_length (model->priv->columns);
@@ -416,8 +459,8 @@ gda_data_model_bdb_get_property (GObject *object,
 
 
 /**
- * gda_data_model_bdb_new
- * @filename:
+ * gda_data_model_bdb_new:
+ * @filename: name of the file containing the database
  * @db_name: the name of the database within @filename, or %NULL
  *
  * Creates a new #GdaDataModel object to access the contents of the Berkeley DB file @file,
@@ -439,12 +482,12 @@ gda_data_model_bdb_new (const gchar *filename, const gchar *db_name)
 }
 
 /**
- * gda_data_model_bdb_get_errors
+ * gda_data_model_bdb_get_errors:
  * @model: a #GdaDataModelBdb object
  *
  * Get the list of errors which have occurred while using @model
  *
- * Returns: a read-only list of #GError pointers, or %NULL if no error has occurred
+ * Returns: (element-type GObject.Error) (transfer none): a read-only list of #GError pointers, or %NULL if no error has occurred
  */
 const GSList *
 gda_data_model_bdb_get_errors (GdaDataModelBdb *model)
@@ -456,7 +499,7 @@ gda_data_model_bdb_get_errors (GdaDataModelBdb *model)
 }
 
 /**
- * gda_data_model_bdb_clean_errors
+ * gda_data_model_bdb_clean_errors:
  * @model: a #GdaDataModelBdb object
  *
  * Reset the list of errors which have occurred while using @model
@@ -556,7 +599,7 @@ move_cursor_at (GdaDataModelBdb *model, gint row)
 		memset (&data, 0, sizeof data);
 		ret = dbpc->c_get (dbpc, &key, &data, DB_FIRST);
 		if (ret) {
-			add_error (model, db_strerror (ret));
+			add_error (model, _db_strerror (ret));
 			return FALSE;
 		}
 		model->priv->cursor_pos = 0;
@@ -569,7 +612,7 @@ move_cursor_at (GdaDataModelBdb *model, gint row)
 			memset (&data, 0, sizeof data);
 			ret = dbpc->c_get (dbpc, &key, &data, DB_NEXT);
 			if (ret) {
-				add_error (model, db_strerror (ret));
+				add_error (model, _db_strerror (ret));
 				return FALSE;
 			}
 			model->priv->cursor_pos ++;
@@ -581,7 +624,7 @@ move_cursor_at (GdaDataModelBdb *model, gint row)
 			memset (&data, 0, sizeof data);
 			ret = dbpc->c_get (dbpc, &key, &data, DB_PREV);
 			if (ret) {
-				add_error (model, db_strerror (ret));
+				add_error (model, _db_strerror (ret));
 				return FALSE;
 			}
 			model->priv->cursor_pos --;
@@ -609,7 +652,7 @@ gda_data_model_bdb_get_value_at (GdaDataModel *model, gint col, gint row, GError
 		tmp = g_strdup_printf (_("Column %d out of range (0-%d)"), col, imodel->priv->n_columns - 1);
 		add_error (imodel, tmp);
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_COLUMN_OUT_OF_RANGE_ERROR,
-			     tmp);
+			      "%s", tmp);
 		g_free (tmp);
 		return NULL;
 	}
@@ -621,9 +664,9 @@ gda_data_model_bdb_get_value_at (GdaDataModel *model, gint col, gint row, GError
         memset (&data, 0, sizeof data);
 	ret = imodel->priv->dbpc->c_get (imodel->priv->dbpc, &key, &data, DB_CURRENT);
 	if (ret) {
-		add_error (imodel, db_strerror (ret));
+		add_error (imodel, _db_strerror (ret));
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
-			     db_strerror (ret));
+			      "%s", _db_strerror (ret));
 		return NULL;
 	}
 
@@ -646,7 +689,7 @@ gda_data_model_bdb_get_value_at (GdaDataModel *model, gint col, gint row, GError
 					gchar *str;
 					str = g_strdup_printf (_("Custom BDB model implementation is not complete: "
 								 "the '%s' method is missing"), "get_key_part");
-					g_warning (str);
+					g_warning ("%s", str);
 					g_free (str);
 				}
 				value = gda_value_new_null ();
@@ -679,7 +722,7 @@ gda_data_model_bdb_get_value_at (GdaDataModel *model, gint col, gint row, GError
 					gchar *str;
 					str = g_strdup_printf (_("Custom BDB model implementation is not complete: "
 								 "the '%s' method is missing"), "get_data_part");
-					g_warning (str);
+					g_warning ("%s", str);
 					g_free (str);
 				}
 				value = gda_value_new_null ();
@@ -742,7 +785,7 @@ gda_data_model_bdb_set_value_at (GdaDataModel *model, gint col, gint row, const
 		tmp = g_strdup_printf (_("Column %d out of range (0-%d)"), col, imodel->priv->n_columns - 1);
 		add_error (imodel, tmp);
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_COLUMN_OUT_OF_RANGE_ERROR,
-			     tmp);
+			      "%s", tmp);
 		g_free (tmp);
 		return FALSE;
 	}
@@ -842,9 +885,9 @@ gda_data_model_bdb_set_values (GdaDataModel *model, gint row, GList *values, GEr
         memset (&data, 0, sizeof data);
 	ret = imodel->priv->dbpc->c_get (imodel->priv->dbpc, &key, &data, DB_CURRENT);
 	if (ret) {
-		add_error (imodel, db_strerror (ret));
+		add_error (imodel, _db_strerror (ret));
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
-			     db_strerror (ret));
+			      "%s", _db_strerror (ret));
 		return FALSE;
 	}
 
@@ -854,7 +897,7 @@ gda_data_model_bdb_set_values (GdaDataModel *model, gint row, GList *values, GEr
 
 	if (key_modified) {
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_FEATURE_NON_SUPPORTED_ERROR,
-			     _("Key modification is not supported"));
+			      "%s", _("Key modification is not supported"));
 		return FALSE;
 	}
 
@@ -905,9 +948,9 @@ gda_data_model_bdb_set_values (GdaDataModel *model, gint row, GList *values, GEr
 	if (!key_modified) {
 		ret = imodel->priv->dbpc->c_put (imodel->priv->dbpc, &key, &data, DB_CURRENT);
 		if (ret) {
-			add_error (imodel, db_strerror (ret));
+			add_error (imodel, _db_strerror (ret));
 			g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
-				     db_strerror (ret));
+				      "%s", _db_strerror (ret));
 			return FALSE;
 		}
 	}
@@ -948,9 +991,9 @@ gda_data_model_bdb_append_row (GdaDataModel *model, GError **error)
 
 	ret = imodel->priv->dbp->put (imodel->priv->dbp, NULL, &key, &data, 0);
 	if (ret) {
-		add_error (imodel, db_strerror (ret));
+		add_error (imodel, _db_strerror (ret));
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
-			     db_strerror (ret));
+			      "%s", _db_strerror (ret));
 		return -1;
 	}
 
@@ -975,9 +1018,9 @@ gda_data_model_bdb_remove_row (GdaDataModel *model, gint row, GError **error)
 
 	ret = imodel->priv->dbpc->c_del (imodel->priv->dbpc, 0);
 	if (ret) {
-		add_error (imodel, db_strerror (ret));
+		add_error (imodel, _db_strerror (ret));
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
-			     db_strerror (ret));
+			      "%s", _db_strerror (ret));
 		return FALSE;
 	}
 
